// Bernard Lidicky - 2005 - GPL - project Pexeso
package cz.nipax.hippo.pexeso;

/**
 * The basic class for all other AI classes. Defines basic interface for AI.
 * See {@link Playground} for description of calls between AI and rest of the game.
 * Object of this class shouldn't be created od used since this class
 * doesn't implement any inteligence.
 * It is recommended to check {@link #m_killed} and <code>m_pg.isEnd()</code>
 * for end of AI's living.
 */
public class AI {


	/**
	 * Sleeptime between moves. For better feeling for human.
	 */
	protected static int MOVESLEEP;
	
	/**
	 * Set sleep time.
	 * @param time in miliseconds
	 */
	public static void setSleep(int time) {
		MOVESLEEP = time;
	}
	
	/**
	 * Playground where this AI is used. It is important for calls back
	 * to <code>Playground</code> using {@link Playground#wantturn}
	 */
	protected Playground m_pg;     // local pointer to playground
	

	/**
	 * Determines state of AI. If <code>m_killed == true</code> the 
	 * AI should stop all computation and don't call any funcions
	 * from {@link #m_pg}. 
	 * @see #kill
	 */
	protected boolean    m_killed; // if the AI is killed
	
	/**
	 * Simple constructior that only sets {@link #m_pg}.
	 * @param pg the Playground thi AI is playing on
	 */
	public AI(Playground pg){
		m_pg = pg;
		MOVESLEEP = 1000;
	}

	/**
	 * Determines weather {@link #println} will write
	 * anything to output.
	 */
	final static boolean DEBUG = false; // to disable messages
	
	/**
	 * Inteligent printing message. It is possible
	 * to disable all messages in one place at once.
	 * @param s the message to print
	 */
	public void println(String s) {
		if (DEBUG) System.out.println(getname() + ": " + s);
	}
	
	/**
	 * Gets name of AI. Should be overriden in child classes.
	 * @return string with name
	 */
	public String getname(){
		return "Base AI";
	}
		
	/**
	 * If user clicks on any card, this event is generated.
	 * Every time this AI is on turn all user's clicks are
	 * passed here.
	 * @param x x coordinate of click
	 * @param y y coortinate of click
	 */
	public void userinput(int x, int y) {
	}
	
	/**
	 * Notification from {@link Playground} that it is our turn. Now the AI has two turns.
	 * It is possible to make two moves directly from this function or wait for second
	 * turn to function {@link #revealed}. If the AI is successfull and guess two same
	 * positions, this method is called again.
	 */
	public void onturn() {
	}
	
	/**
	 * Notification from {@link Playground} that it card in playground was revealed.
	 * AI may want to remember revealed cards somewere. This method is called everytime
	 * any card is revealed. 
	 * @param x  coordinate of card 
	 * @param y  coordinate of card 
	 * @param ID ID of card
	 */
	public void revealed(int x, int y, int ID) {
	}

	/**
	 * Notify AI, that position is now invalid. Notification 
	 * for one position may occure multipletimes. AI shouldn't
	 * do any strong computaion here. 
	 * @param x coordinate of now invalid position
	 * @param y coordinate of now invalid position
	 */
	public void ivalidate(int x, int y) {
	}

	/**
	 * Notify AI, that it should end now and no longer disturb anyone.
	 * {@link Playground} should take no more input {@link Playground#wantturn}
	 * from killed AI.
	 * This sets {@link #m_killed} to <code>true</code>
	 */
	public void kill()
	{
		m_killed = true;
	}
}
