// Bernard Lidicky - 2005 - GPL - project Pexeso
package cz.nipax.hippo.pexeso;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.*;


/**
 * Holds main window and playground. Takes care about menu, menu callback,
 * creates {@link Playground} and {@link PlaygroundComponent}. 
 */
public class MainDlg implements ActionListener {

	// Gui items
	
	/** The main frame of application. */
	JFrame    frame;
	
	/** Main menu item. */
	JMenuItem itmQuit;
	/** Main menu item. */
	JMenuItem itmAbout;
	/** Main menu item. */
	JMenuItem itmNew;

	// Game items
	/** The game state. */
	Playground pg;
	/** Component for displaying game status. */
	PlaygroundComponent pgc;

	/** Constructs and inicializes {@link #pg}. */
	public MainDlg() {
		pg = new Playground();
		pg.init(6,5);
		pg.add_AI(new AI_normal(pg), new AI_normal(pg));
	}
	
	/**
	 * Callbacks for menu. Handles quit programm, about dialog and
	 * new game dialog.
	 */
	public void actionPerformed(ActionEvent e) {
		Object src = e.getSource();
		
		if (src.equals(itmQuit)) {
			//System.out.println("MainDlg: Quit");	
			System.exit(0);
		}
		
		if (src.equals(itmAbout)) {
			//System.out.println("MainDlg: About");
			
			AboutDlg dlgAbout = new AboutDlg();
			dlgAbout.createGUI(frame);
		}
		
		if (src.equals(itmNew)) {
			//System.out.println("MainDlg: New game");

			NewGameDlg dlg = new NewGameDlg();
			dlg.createGUI(frame, pg);
		}
		
		
	}
	
	/**
	 * Constructs main dialog. Should be called in gui thread.
	 * After construction of dialog shows new game dialog.
	 */
	public void createGUI() {

		
		frame = new JFrame("Pexeso");
		
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		pgc = new PlaygroundComponent(pg);
		frame.getContentPane().add(pgc);
		
		// Creating menu bar
		JMenuBar mb = new JMenuBar();
		
		// Section Game
		JMenu menu = new JMenu("Game");
		itmQuit = new JMenuItem("Quit");
		itmQuit.addActionListener(this);
		itmNew = new JMenuItem("New");
		itmNew.addActionListener(this);
		menu.add(itmNew);
		menu.add(itmQuit);
		mb.add(menu);
		
		// Section Help
		menu = new JMenu("Help");
		itmAbout = new JMenuItem("About");
		itmAbout.addActionListener(this);
		menu.add(itmAbout);
		//menu.add(new JSeparator());
		mb.add(menu);
		
		frame.setJMenuBar(mb);
		frame.pack();
		frame.setVisible(true);
		
	
		
		// this has nothing to do with the GUI :-)
	//	pg.start();
		NewGameDlg dlg = new NewGameDlg();
		dlg.createGUI(frame, pg);
	}
	
}
