// Bernard Lidicky - 2005 - GPL - project Pexeso
package cz.nipax.hippo.pexeso;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Creates modal dialog to select players for new game. Lets user to select players (AI) and then
 * calls <code>Playground.init</code>, <code>Playground.add_AI</code> and <code>Playground.start</code>
 * Also remembers last slelection for next time of creating this dialog.
 * <p>
 * Kaffe has problem with showing combo boxes (I don't know why). They apear OK,
 * but changing the value isn't what one would expect. The popup window somehow floats.
 *      
 * @see Playground#init
 * @see Playground#add_AI
 * @see Playground#start
 * 
 * 
 */
public class NewGameDlg {

	/**
	 * Remembers selection of first player's AI. It is index
	 * of selection in {@link #cbox1}
	 */
	private static int cbox1_sel = 0;

	/**
	 * Remembers selection of second player's AI. It is index
	 * of selection in {@link #cbox2}
	 */
	private static int cbox2_sel = 0;

	/**
	 * Remembers beast mode.
	 */
	private static boolean beastmode = false;

	/**
	 * Remembers turn switch mode
	 */
	private static boolean turnswitch = false; 
	
	/**
	 * The dialog window. 
	 */
	private JDialog dialog;
	
	/**
	 * The combobox for selecting first's player AI.
	 */
	private JComboBox cbox1;
	
	/**
	 * The combobox for selecting second's player AI.
	 */
	private JComboBox cbox2;
	
	/**
	 * Checkbox for beastmode.
	 */
	private JCheckBox chbox_beast;
	
	/**
	 * Checkbox for turn switch
	 */
	private JCheckBox chbox_ts;

	
	/**
	 * The playground for which this dialog selects players.
	 * @see Playground
	 */
	private Playground pg;

	/**
	 * Simple constructor that set all to null.
	 */
	public NewGameDlg() {
		dialog = null;
		cbox1 = null;
		cbox2 = null;
		pg = null;
		chbox_beast = null;
		chbox_ts = null;
	}

	/** 
	 * Creates AI object from selection of combobox. The order of 
	 * AI names must be synchronized with {@link #createCombo}
	 * @param ID from which AI is created
	 * @return newly created AI object
	 * @see AI
	 */
	private AI getAI(int ID) {
		     if (ID == 0) return new AI_human(pg);
		else if (ID == 1) return new AI_random(pg);
		else if (ID == 2) return new AI_lhd(pg,0);
		else if (ID == 4) return new AI_lhd(pg,1);
		else if (ID == 3) return new AI_forgetting(pg);
		else if (ID == 5) return new AI_normal(pg);
		else if (ID == 6) return new AI_enhanced(pg);
		else if (ID == 7) return new AI_snc(pg);
		else return new AI_random(pg);
	}

	/**
	 * Creates new combobox object filled with AI names.
	 * The order for adding AI is relevatn for {@link #getAI}
	 * @return new combobox object
	 */
	private JComboBox createCombo() {
		JComboBox cbox = new JComboBox();
		cbox.addItem("Human");
		cbox.addItem("AI - random");
		cbox.addItem("AI - more cursed");
		cbox.addItem("AI - less cursed");
		cbox.addItem("AI - forgetting");
		cbox.addItem("AI - normal");
		cbox.addItem("AI - enhanced");
		cbox.addItem("AI - still not cheating :-)");
		if (Main.OLD_CP) { // Kaffe shows combo box badly - last lines are missing
		}
		cbox.setLightWeightPopupEnabled(true);
		cbox.setEnabled(true);
		return cbox;
	}
	
	/**
	 * Creates and shows the modal dialog. Constructs all buttons and
	 * comboboxes. Includes implementation of action listeners for
	 * buttons.
	 * @param frame the main app frame, parent of this dialog
	 * @param pg_in Where to add new AI and start new game. If user
	 *              clicks cancel button, no method from <code>pg_in</code>
	 *              is called.
	 */
	public void createGUI(JFrame frame, Playground pg_in) {

		pg = pg_in;
	
		// Combos and VS
		cbox1 = createCombo();
		cbox1.setToolTipText("First player");
		cbox1.setSelectedIndex(cbox1_sel);
		
		cbox2 = createCombo();
		cbox2.setToolTipText("Second player");
		cbox2.setSelectedIndex(cbox2_sel);
		
		JLabel vs = new JLabel();
		vs.setText("VS");
	
		// combos and VS pane
		JPanel selectPane = new JPanel(new FlowLayout(FlowLayout.CENTER,5,0));
		selectPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		selectPane.add(cbox1);
		selectPane.add(vs);
		selectPane.add(cbox2);

	
		// checkboxes
		chbox_beast = new JCheckBox("Beast mode.", beastmode);
		chbox_beast.setToolTipText("Randomly mixes cards during game.");
		
		chbox_ts  = new JCheckBox("Turn switch.", turnswitch);
		chbox_ts.setToolTipText("Switches revealed cards.");
		
		
		// Buttons - cancel and OK
		JButton butt_cancel = new JButton("Cancel");
		butt_cancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dialog.setVisible(false);
				dialog.dispose();
				dialog = null;
			}
		});

		JButton butt_ok = new JButton("Start game");
		butt_ok.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			
				// saving info
				beastmode = chbox_beast.isSelected();
				turnswitch = chbox_ts.isSelected();
				cbox1_sel  = cbox1.getSelectedIndex();
				cbox2_sel  = cbox2.getSelectedIndex();
				
				// game start
				pg.init(6,5);
				pg.add_AI(getAI(cbox1_sel), getAI(cbox2_sel));
				pg.setSwitchturn(turnswitch);
				pg.setBeastmode(beastmode);
				pg.start();
				
				dialog.setVisible(false);
				dialog.dispose();
				dialog = null;
			}
		});

	
		
		// Buttons pane	
		JPanel buttonPane = new JPanel(new FlowLayout(FlowLayout.CENTER,5,0));
		buttonPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		buttonPane.add(butt_ok);
		buttonPane.add(butt_cancel);

		// Content pane
		JPanel contentPane = new JPanel();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));
		contentPane.add(selectPane, new Float(JPanel.CENTER_ALIGNMENT));
		contentPane.add(chbox_beast, new Float(JPanel.CENTER_ALIGNMENT));
		contentPane.add(chbox_ts, new Float(JPanel.CENTER_ALIGNMENT));
		contentPane.add(buttonPane, new Float(JPanel.RIGHT_ALIGNMENT));
		contentPane.setOpaque(true);
		
		// modal dialog
		dialog = new JDialog(frame, "Start new game", true);
		dialog.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		dialog.setContentPane(contentPane);
		dialog.setLocationRelativeTo(frame);
		dialog.pack();
		dialog.setVisible(true);
	}
	

}
