// Bernard Lidicky - 2005 - GPL - project Pexeso
package cz.nipax.hippo.pexeso;

import java.awt.Container;
import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.io.*;
import javax.swing.*;
import java.awt.image.VolatileImage;
import java.awt.image.BufferedImage;

//import gnu.java.awt.peer.gtk.GtkImage;

/**
 * PlaygroundComponent creates playground and gets user input. It requires
 * to have valid {@link Playground} in constructor to know what to display.
 * Playground should call repaint if gamestate changes.
 * <p>
 * If any AI wants to receive input from user, it needs to register.
 * Just call {@link #registerAI}. Note that PlaygroundComponent
 * stores only one AI. So if game is with more AIs that want user input,
 * {@link #registerAI} everytime AI on turn is changed.
 * This call is done by {@link Playground#switchplayer}, so AI class
 * doesn't have to do it.
 */
public class PlaygroundComponent extends JComponent implements MouseListener{
	
	/** Enables debug messages. */
	final static boolean DEBUG = false; 
	
	/**
	 * Inteligent message printing. It is possible
	 * to disable all messages in one place - for some
	 * kind of final release
	 */
	public void println(String s) {
		if (DEBUG) System.out.println("PlaygroundComponent: " + s);
	}
	

	/** Local reference to main {@link Playground} */
	private Playground m_pg; 

	// 
	// Font    m_font;

	/** Currently active AI. Needed to know where to pass user input. 
	 * May be <code>null</code>.*/
	AI      m_ai; 

	/** All graphic and dimensions of graphics */
	PGTheme m_theme;

	/** Off screen image of game. For better graphic performance */
	private VolatileImage m_back_vol;
	private Image m_back;
	
	
	/** 
	 * Perpares whole class for use.
	 * Loads images of pexesos :-) Also register self into
	 * {@link Playground} via {@link Playground#setPGC}.
	 * @param pg what playground to display
	 */
	PlaygroundComponent(Playground pg) {
		
		addMouseListener(this);
		
		m_ai = null;
		
		// local copy and register as a GUI
		m_pg = pg;
		m_pg.setPGC(this);
		
		// create font - nefunguje v kaffe
		/*
		File f = new File("FreeMonoOblique.ttf");
		//m_font = Font.createFont(Font.TRUETYPE_FONT, f);
		try {
		m_font = Font.createFont(Font.TRUETYPE_FONT, new FileInputStream(f));
		} catch (IOException io) {}
		catch(FontFormatException ff) {};
		*/
		// m_font = new Font("Arial", Font.PLAIN, 20);
		
		setTheme(new PGTheme());

		m_back = null;

		setOpaque(true);
	}
	

	
	/**
	 * Set current AI. Changes {@link #m_ai}.
	 * @param ai the current ai.
	 */
	public void registerAI(AI ai) {
		m_ai = ai;
	}

	/**
	 * Chenges current theme. Owerwrites {@link #m_theme}. 
	 * @param theme new theme
	 */
	public void setTheme(PGTheme theme) {
		m_theme = theme;
	}
	
	public void createback() {
	
		m_back_vol = createVolatileImage(550, 500);
		
		if (m_back_vol == null) {
			
			System.err.println("Unable to create accelerated buffer.");
			
			  m_back = new BufferedImage(550,500,BufferedImage.TYPE_INT_RGB);
			// GtkImage gi = new GtkImage(null, null);
			// GtkImage gi = new GtkImage();
			// gi.setDimensions(550, 500);
			// m_back = gi;
			// Toolkit toolkit = Toolkit.getDefaultToolkit();
			// m_back = toolkit.getImage("back.png");
		} else {
			m_back = m_back_vol;
		}
		
		if (m_back == null) {
			System.err.println("Unable to create an offscreen image buffer!");
			System.exit(1);
		}
		
		println("Back created");
	}
	
	/**
	 * Set default size of playground.
	 */
	public Dimension getPreferredSize() {
		return new Dimension(550,500);
	}	
	
    
	public void paintComponent(Graphics g) {
		
		Rectangle r = g.getClipBounds();

		// simple clear rect doesn't work in Kaffe - celars to black :-(
//		 g.clearRect(r.x, r.y, r.width, r.height);
//		 
		if (m_back_vol != null) {		 
		 	int returnCode = m_back_vol.validate(getGraphicsConfiguration());
		 	if (returnCode == VolatileImage.IMAGE_RESTORED ||
		 	    returnCode == VolatileImage.IMAGE_INCOMPATIBLE) {	 
				 // Contents need to be restored || recreated
				 println("Invalidation");
				 paint_offscreen(false, null);
			     }
		}
		

		if (Main.OLD_CP) {
			paint_offscreen(false, g);
		}
		else {
			if (m_back == null) paint_offscreen(false, null);
			g.drawImage(m_back, 0, 0, this);
		}
	}
	
	/**
	 * Paints playground and game statistic. 
	 * Performs full redraw but without clearing backgorund.
	 * Uses cliping for better perfomance.
	 */
	//public void paint(Graphics g) {
	public void paint_offscreen(boolean updateonly, Graphics gr) {

		if (m_back == null) createback();
		
		println("paint_offscreen");
		
		
		if (m_back_vol != null && m_back_vol.contentsLost()) updateonly = false;
		
		if (m_back_vol != null && m_back_vol.validate(getGraphicsConfiguration()) ==
				VolatileImage.IMAGE_INCOMPATIBLE) 
		{
			// old m_back  doesn't work with new GraphicsConfig; re-create it
			createback();  // recreate the image
			updateonly = false;
		}
		
		
		Graphics g;
		if (gr != null) g = gr;
		else g = m_back.getGraphics();
		
		if (!updateonly) g.clearRect(0,0,550,500);
		if (Main.OLD_CP) g.setColor(Color.red);
		else g.setColor(Color.black);
		
		// Game end
		if (m_pg.isEnd()) {
			switch (m_pg.getWinner()) {
				case 0:
					g.drawString("First player won!",20, 25);
					break;
				case 1:
					g.drawString("Second player won!",20, 25);
					break;
				case 2:
					g.drawString("No winner",20, 25);
					break;
				}
				g.drawString("score 1: " + m_pg.getScore(0) + " 2: "  + m_pg.getScore(1), 20, 50);
		}
		// In game screen
		else {
			if (updateonly) g.clearRect(0,0,550,38);
			
			g.drawString("Score " + m_pg.getScore(0) + ":"  + m_pg.getScore(1) +
			             "  On turn: " + m_pg.getCPName(), 20, 25);
			
			for(int x = 0, sx = 20; x < m_pg.getX(); x++, sx+=m_theme.getW()+m_theme.getSpace())  {
				for(int y = 0, sy = 40; y < m_pg.getY(); y++, sy+=m_theme.getH()+m_theme.getSpace()) {
					
					int ID = m_pg.getID(x, y);
					if (m_pg.checkDirty(x, y) || (!updateonly && ID != -1)) {
					
						if (updateonly)
							g.clearRect(sx, sy, m_theme.getW(), m_theme.getH());
						
						if (ID == -1) continue;
						Image img = m_theme.getImage(ID);
						if (img != null) drawImage_true(g, img, sx, sy);
						else g.drawString(ID+"",sx, sy+20);
					}
				}
			}
		}

		if (gr == null)	g.dispose();
	}

	/**
	 * This should be caled from {@link Playground} if repaint is wanted.
	 * @param update_only if only few cards chenged visibility - for incremental
	 *                    painting and thus better performance. This is critical
	 *                    for Kaffe if you want to be able to play th egame.
	 */                    
	public void repaint_later(boolean update_only) {
	
		
		if (!Main.OLD_CP) paint_offscreen(update_only, null);
		repaint();
		
	}
	
	/**
	 * Graphics.drawImage may return false and then no image is drawn. Usually
	 * repeted paint with some delay correct this. The problem was observed with
	 * sun java 1.5. classpath worked with no problem. It looks like the picture
	 * need some time to get ready before drawing. It happens for one picture
	 * only once.
	 */
	private void drawImage_true(Graphics g, Image img, int x, int y)
	{
		if (g == null || img == null) return;
		
		while (g.drawImage(img, x, y, null) == false) {
			println("drawfail!");
			// little sleep may correct it
			try {Thread.sleep(10);}
			catch(InterruptedException e) {}
		}
	}
	
	
	/**
	 * Computes field where user has clicked and sends it to {@link #m_ai} via
	 * {@link AI#userinput}.
	 */
	public  void	 mouseClicked(MouseEvent event) {
		 println("Clicked: "+event.getPoint());

		 if (m_ai == null) return;
		 
		 int x = event.getX();
		 int y = event.getY();
	 
//		 x -= 20;
//		 y -= 40;
//		 if ( x % (m_theme.getW()+m_theme.getSpace()) > 
	 
		for(int i = 0, sx = 20; i < m_pg.getX(); i++, sx+=m_theme.getW()+m_theme.getSpace())  {
			for(int j = 0, sy = 40; j < m_pg.getY(); j++, sy+=m_theme.getH()+m_theme.getSpace()) {
				if (sx < x && sy < y && 
				    sx+m_theme.getW() > x && sy+m_theme.getH() > y) {
					m_ai.userinput(i, j);
				 	println("input "+i+" "+j);
				}
			}
		}
	}

/** Empty function - just for MouseListener interface. */
public  void	 mouseEntered(MouseEvent event) {}
/** Empty function - just for MouseListener interface. */
public  void	 mouseExited(MouseEvent event) {}
/** Empty function - just for MouseListener interface. */
public  void	 mousePressed(MouseEvent event) {}
/** Empty function - just for MouseListener interface. */
public  void	 mouseReleased(MouseEvent event) {}

}
